import { Controller } from '@hotwired/stimulus';

export default class extends Controller {
  static targets = ['image', 'openIcon'];

  connect() {
    this.isOpen = this.element.dataset.isOpen === 'true';
    this.updateOpenStatusIcon();
    this.parentIds = this.element.dataset.parentIds;
    this.parentId = this.parentIds.split(',').pop();
    this.closeImg = this.element.dataset.closeImg;
    this.openImg = this.element.dataset.openImg;

    this.toggleSubrowsHandler = this.toggleSubrows.bind(this);
    this.closeIfParentClosedHandler = this.closeIfParentClosed.bind(this);
    this.element.addEventListener('click', this.toggleSubrowsHandler);
    this.element.closest('table').addEventListener('subrows:close', this.closeIfParentClosedHandler);
  }

  disconnect() {
    this.element.removeEventListener('click', this.toggleSubrowsHandler);
    this.element.closest('table')?.removeEventListener('subrows:close', this.closeIfParentClosedHandler);
  }

  toggleSubrows() {
    if (this.isOpen) {
      this.close();
      this.element.dispatchEvent(
        new CustomEvent('subrows:close', {
          bubbles: true,
          detail: { parentId: this.parentId },
        })
      );
    } else {
      this.open();
    }
  }

  updateSubrows() {
    this.subrows = document.querySelectorAll(`tr[data-parent-id="${this.parentId}"]`);
  }

  close() {
    this.isOpen = false;
    this.updateSubrows();
    this.updateOpenStatusIcon();
    this.subrows.forEach(subrow => {
      subrow.classList.add('hidden');
    });
    this.imageTarget.src = this.closeImg;
  }

  open() {
    this.isOpen = true;
    this.updateOpenStatusIcon();
    this.updateSubrows();
    this.subrows.forEach(subrow => {
      subrow.classList.remove('hidden');
    });
    this.imageTarget.src = this.openImg;
  }

  updateOpenStatusIcon() {
    const classes = this.isOpen ? 'fa fa-chevron-down' : 'fa fa-chevron-right';
    const newElementHTML = `<i data-toggle-subrows-target="openIcon" class="${classes}"></i>`;
    this.openIconTarget.innerHTML = newElementHTML;
  }

  closeIfParentClosed(event) {
    if (event.detail.parentId !== this.parentId && this.isOpen && this.parentIds.includes(event.detail.parentId)) {
      this.close();
    }
  }
}
